/* just some information about this file */
#define CLP_VERSION	"1.0.4"
#define CLP_DATE "29-Jan-2004"

/* structures and typedefs */

/** @file
* @anchor CommandLineParameters
* Fault-tolerant structure to store the values of all command line arguments.
*
* This structure provides a fault-tolerant way to store the values of all command line arguments. The
* values are set to their default values during the creation of the structure via @ref CLP_create.
* The memory allocation of the members of the structure is managed automatically.
*
* The structure is created with @ref CLP_create, destroyed with @ref CLP_destroy and displayed with
* @ref CLP_display. With CLP_set<i>nameOfVariable</i> and CLP_get<i>nameOfVariable</i> the values of
* the individual members of the structure @ref CommandLineParameters could be set or gotten.
*
* Whenever a string is set the provided string is @b copied via @c strncpy. Hence, the string provided
* as an argument has to be freed by the caller of the function.
* 
* <B>Description of the members:</B><BR><BR>
* @c outputFile - string containing the name of the output file.<BR>
* @c reactionFile - string storing the name of the reaction file.<BR>
* @c errorLogFile - string storing the name of the SMILES error log file.<BR>
* <P>
* @author Uli Fechner
* @version 05/13/2003 - Uli Fechner - v1.0.0 - initial release
* @version 24/11/2003 - Uli Fechner - v1.0.1 - modified according to the needs of retroflux
* @version 03/12/2003 - Uli Fechner - v1.0.2 - added the member @c errorLogFile
* @version 07/01/2004 - Uli Fechner - v1.0.3 - added the member @c filterFile
* @version 29/01/2004 - Tina Grabowski - v1.0.4 - added the member @c smartsFile and the corresponding
*                       functions @ref CLP_getSmartsFile and @ref CLP_setSmartsFile; consideration of @c
*                       smartsFile in @ref CLP_create, @ref CLP_display and @ref CLP_destroy
* @@code
*/
typedef struct
{
	char* outputFile;
	char* reactionFile;
	char* filterFile;
	char* smartsFile;
	char* errorLogFile;
} CommandLineParameters;
/** @endcode */

/** A pointer to structure @ref CommandLineParameters is assigned the name @c CLP_Ptr. */
typedef CommandLineParameters* CLP_Ptr;

/* function prototypes */

CLP_Ptr CLP_create( void );

void CLP_destroy( CLP_Ptr clpPtr );

void CLP_display( const CLP_Ptr clpPtr, FILE* outputStream );

void CLP_setOutputFile( const CLP_Ptr clpPtr, const char* const name );

char* CLP_getOutputFile( const CLP_Ptr clpPtr );

void CLP_setReactionFile( const CLP_Ptr clpPtr, const char* const name );

char* CLP_getReactionFile( const CLP_Ptr clpPtr );

void CLP_setErrorLogFile( const CLP_Ptr clpPtr, const char* const name );

char* CLP_getErrorLogFile( const CLP_Ptr clpPtr );

void CLP_setFilterFile( const CLP_Ptr clpPtr, const char* const name );

char* CLP_getFilterFile( const CLP_Ptr clpPtr );

void CLP_setSmartsFile( const CLP_Ptr clpPtr, const char* const name );

char* CLP_getSmartsFile( const CLP_Ptr clpPtr );

/* functions */

/** Creates a @ref CommandLineParameters structure.
*
* The structure @ref CommandLineParameters is created. The memory of the structure itself and the members
* of the structure is allocated automatically. The values of the individual command line parameters are
* set to default values as written in the help text.
* @retval CLP_Ptr pointer on the newly created structure @ref CommandLineParameters
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
* @version 24/11/2003 - Uli Fechner - adjusted to the needs of flux
* @version 03/12/2003 - Uli Fechner - added support for the member @c errorLogFile
* @version 07/01/2004 - Uli Fechner - added support for the member @c filterFile
* @version 29/01/2004 - Tina Grabowski - added support for the member @c smartsFile
*/
CLP_Ptr CLP_create( void )
{
	CLP_Ptr clpPtr;
	if( !( clpPtr = calloc( 1, sizeof( CommandLineParameters ) ) ) )
		MemoryError( "CLP_create", "clpPtr" );

	CLP_setOutputFile( clpPtr, NULL );
	CLP_setReactionFile( clpPtr, NULL );
	CLP_setFilterFile( clpPtr, NULL );
	CLP_setSmartsFile( clpPtr, NULL );
	CLP_setErrorLogFile( clpPtr, NULL );

	return clpPtr;
}

/** Destroys a structure @ref CommandLineParameters.
*
* The structure @ref CommandLineParameters the pointer @c clpPtr refers to is destroyed. All
* allocated memory of the structure and its members is automatically freed.
*
* @attention
* The two strings (@c outputFile and @c catsTypeExpression) are freed here, too.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that should be destroyed
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
* @version 24/11/2003 - Uli Fechner - adjusted to the needs of retroflux
* @version 07/01/2004 - Uli Fechner - added support for the member @c filterFile
* @version 29/01/2004 - Tina Grabowski - added support for the member @c smartsFile
*/
void CLP_destroy( CLP_Ptr clpPtr )
{
	if( clpPtr != NULL )
	{
		if( CLP_getOutputFile( clpPtr ) != NULL )
			free( CLP_getOutputFile( clpPtr ) );
		if( CLP_getReactionFile( clpPtr ) != NULL )
			free( CLP_getReactionFile( clpPtr ) );
		if( CLP_getFilterFile( clpPtr ) != NULL )
			free( CLP_getFilterFile( clpPtr ) );
		if( CLP_getSmartsFile( clpPtr ) != NULL )
			free( CLP_getSmartsFile( clpPtr ) );
		free( clpPtr );
	}
	else
	{
		fprintf( stderr, "\n\nERROR: Function 'CLP_destroy' is not able to destroy a structure\n" );
		fprintf( stderr, "'CommandLineParameters' that has not been created before!\n" );
		AbortProgram;
	}
}

/** Displays a structure @ref CommandLineParameters.
*
* The structure @ref CommandLineParameters the pointer @c clpPtr refers to is displayed on the FILE* @c
* outputStream.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that should be displayed
* @param outputStream FILE* on the stream the output should be sent to
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
* @version 24/11/2003 - Uli Fechner - adjusted to the needs of flux
* @version 03/12/2003 - Uli Fechner - added support for the member @c errorLogFile
* @version 07/01/2004 - Uli Fechner - added support for the member @c filterFile
* @version 29/01/2004 - Tina Grabowski - added support for the member @c smartsFile
*/
void CLP_display( CLP_Ptr clpPtr, FILE* outputStream )
{	
	fprintf( outputStream, "Output file: %s\n", CLP_getOutputFile( clpPtr ) );
	fprintf( outputStream, "Error log file: %s\n", CLP_getErrorLogFile( clpPtr ) );
	if( clpPtr->filterFile != NULL )
		fprintf( outputStream, "Filter file: %s\n", CLP_getFilterFile( clpPtr ) );
	if( clpPtr->reactionFile != NULL )
		fprintf( outputStream, "Reaction file: %s\n", CLP_getReactionFile( clpPtr ) );
	if( clpPtr->smartsFile != NULL )
		fprintf( outputStream, "Smarts file: %s\n", CLP_getSmartsFile( clpPtr ) );
	fprintf( outputStream, "\n" );
}

/** The name of the output file is set.
*
* @attention
* The provided parameter @c name is copied. @c name is NOT freed afterwards. This has to be done by
* the caller of this function.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @param name name of the output file
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
*/
void CLP_setOutputFile( const CLP_Ptr clpPtr, const char* const name )
{
	if( name != NULL )
	{
		if( !( clpPtr->outputFile = calloc( strlen( name ) + 1, sizeof( char ) ) ) )
			MemoryError( "CLP_setOutputFile", "clpPtr->outputFile" );	
		strncpy( clpPtr->outputFile, name, strlen( name ) + 1 );
	}
	else
		clpPtr->outputFile = NULL;
}

/** The name of the output file is returned.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @retval char name of the output data file
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
*/
char* CLP_getOutputFile( const CLP_Ptr clpPtr )
{
	return clpPtr->outputFile;
}

/** The name of the reaction file is set.
*
* @attention
* The provided parameter @c name is copied. @c name is NOT freed afterwards. This has to be done by
* the caller of this function.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @param name name of the reaction file
* @author Uli Fechner
* @version 24/11/2003 - Uli Fechner - initial release
*/
void CLP_setReactionFile( const CLP_Ptr clpPtr, const char* const name )
{
	if( name != NULL )
	{
		if( !( clpPtr->reactionFile = calloc( strlen( name ) + 1, sizeof( char ) ) ) )
			MemoryError( "CLP_setReactionFile", "clpPtr->reactionFile" );	
		strncpy( clpPtr->reactionFile, name, strlen( name ) + 1 );
	}
	else
		clpPtr->reactionFile = NULL;
}

/** The name of the reaction file is returned.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @retval char name of the reaction data file
* @author Uli Fechner
* @version 24/11/2003 - Uli Fechner - initial release
*/
char* CLP_getReactionFile( const CLP_Ptr clpPtr )
{
	return clpPtr->reactionFile;
}

/** The name of the error log file is set.
*
* @attention
* The provided parameter @c name is copied. @c name is NOT freed afterwards. This has to be done by
* the caller of this function.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @param name name of the error log file
* @author Uli Fechner
* @version 03/12/2003 - Uli Fechner - initial release
*/
void CLP_setErrorLogFile( const CLP_Ptr clpPtr, const char* const name )
{
	if( name != NULL )
	{
		if( !( clpPtr->errorLogFile = calloc( strlen( name ) + 1, sizeof( char ) ) ) )
			MemoryError( "CLP_setErrorLogFile", "clpPtr->errorLogFile" );	
		strncpy( clpPtr->errorLogFile, name, strlen( name ) + 1 );
	}
	else
		clpPtr->errorLogFile = NULL;
}

/** The name of the error log file is returned.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @retval char name of the error log file
* @author Uli Fechner
* @version 03/12/2003 - Uli Fechner - initial release
*/
char* CLP_getErrorLogFile( const CLP_Ptr clpPtr )
{
	return clpPtr->errorLogFile;
}

/** The name of the filter file is set.
*
* @attention
* The provided parameter @c name is copied. @c name is NOT freed afterwards. This has to be done by
* the caller of this function.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @param name name of the filter file
* @author Uli Fechner
* @version 07/01/2004 - Uli Fechner - initial release
*/
void CLP_setFilterFile( const CLP_Ptr clpPtr, const char* const name )
{
	if( name != NULL )
	{
		if( !( clpPtr->filterFile = calloc( strlen( name ) + 1, sizeof( char ) ) ) )
			MemoryError( "CLP_setFilterFile", "clpPtr->filterFile" );	
		strncpy( clpPtr->filterFile, name, strlen( name ) + 1 );
	}
	else
		clpPtr->filterFile = NULL;
}

/** The name of the filter file is returned.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @retval char name of the filter file
* @author Uli Fechner
* @version 07/01/2004 - Uli Fechner - initial release
*/
char* CLP_getFilterFile( const CLP_Ptr clpPtr )
{
	return clpPtr->filterFile;
}

/** The name of the Smarts file is set.
*
* @attention
* The provided parameter @c name is copied. @c name is NOT freed afterwards. This has to be done by
* the caller of this function.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @param name name of the Smarts file
* @author Tina Grabowski
* @version 29/01/2004 - Tina Grabowski - initial release
*/
void CLP_setSmartsFile( const CLP_Ptr clpPtr, const char* const name )
{
	if( name != NULL )
	{
		if( !( clpPtr->smartsFile = calloc( strlen( name ) + 1, sizeof( char ) ) ) )
			MemoryError( "CLP_setSmartsFile", "clpPtr->SmartsFile" );	
		strncpy( clpPtr->smartsFile, name, strlen( name ) + 1 );
	}
	else
		clpPtr->smartsFile = NULL;
}

/** The name of the Smarts file is returned.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @retval char name of the Smarts file
* @author Tina Grabowski
* @version 29/01/2004 - Tina Grabowski - initial release
*/
char* CLP_getSmartsFile( const CLP_Ptr clpPtr )
{
	return clpPtr->smartsFile;
}
