/* just some information about this file */
#define GIVENCLP_VERSION	"1.0.2"
#define GIVENCLP_DATE	"29-Jan-2003"

/* structures and typedefs */

/** @file
* @anchor GivenCommandLineParameters
* Structure to store a simulated boolean value for all command line arguments.
*
* This structure provides a way to store simulated boolean values (@ref BOOLEAN_TRUE or @ref
* BOOLEAN_FALSE) for all command line arguments. It is used to store the information whether a
* specific command line argument is given or not. All values are set to @ref BOOLEAN_FALSE during
* the creation of the structure via @ref GivenClp_create.
*
* The structure is created with @ref GivenClp_create, destroyed with @ref GivenClp_destroy and
* displayed with @ref GivenClp_display. With @ref GivenClp_setOption and @ref GivenClp_getOption
* the values of the individual members of the structure @ref GivenCommandLineParameters could be
* set or gotten.
*
* <B>Description of the members:</B><BR><BR>
*	@c oOptionGiven - indicates whether the @c o option is given at the command line<BR>
* @c rOptionGiven - indicates whether the @c r option is given at the command line<BR>
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - v1.0.0 - initial release
* @version 01/09/2003 - Uli Fechner - v1.0.1 - .adjusted according to the needs of retroflux
* @version 29/01/2004 - Tina Grabowski - v1.0.2 - added the @c member sOptionGiven; corresponding
*                       changes in @ref GivenClp_create, @ref GivenClp_display, @ref
*                       GivenClp_setOption and @ref GivenClp_getOption
* @@code
*/
typedef struct
{
	int oOptionGiven;
	int rOptionGiven;
	int fOptionGiven;
	int sOptionGiven;
} GivenCommandLineParameters;
/** @endcode */

/** A pointer to structure @ref GivenCommandLineParameters is assigned the name @c Given_CLP_Ptr. */
typedef GivenCommandLineParameters* Given_CLP_Ptr;

/* function prototypes */

Given_CLP_Ptr GivenClp_create( void );

void GivenClp_destroy( Given_CLP_Ptr givenClpPtr );

void GivenClp_display( const Given_CLP_Ptr givenClpPtr, FILE* outputStream );

void GivenClp_setOption( const Given_CLP_Ptr givenClpPtr, const int option, const int boolean_value );

int GivenClp_getOption( const Given_CLP_Ptr givenClpPtr, const int option );

/* functions */

/** Creates a @ref GivenCommandLineParameters structure.
*
* The structure @ref GivenCommandLineParameters is created. The memory of the structure is allocated
* automatically. The values of all members are set to @ref BOOLEAN_FALSE.
* @retval Given_CLP_Ptr pointer on the newly created structure @ref GivenCommandLineParameters
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
* @version 24/11/2003 - Uli Fechner - adjusted according to the needs of retroflux
* @version 29/01/2004 - Tina Grabowski - added support for the member @c sOptionGiven
*/
Given_CLP_Ptr GivenClp_create( void )
{
	Given_CLP_Ptr givenClpPtr;
	if( !( givenClpPtr = calloc( 1, sizeof( GivenCommandLineParameters ) ) ) )
		MemoryError( "givenClpPtr", "GivenClp_create" );
	
	givenClpPtr->oOptionGiven = BOOLEAN_FALSE;
	givenClpPtr->rOptionGiven = BOOLEAN_FALSE;
	givenClpPtr->fOptionGiven = BOOLEAN_FALSE;
	givenClpPtr->sOptionGiven = BOOLEAN_FALSE;
	
	return givenClpPtr;
}

/** Destroys a @ref GivenCommandLineParameters structure.
*
* The structure @ref GivenCommandLineParameters the pointer @c givenClpPtr refers to is destroyed. The
* allocated memory of the structure is automatically freed.
*
* @param givenClpPtr pointer on the structure @ref GivenCommandLineParameters that should be destroyed
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
*/
void GivenClp_destroy( Given_CLP_Ptr givenClpPtr )
{
	if( givenClpPtr != NULL )
		free( givenClpPtr );
	else
	{
		fprintf( stderr, "\n\nERROR: Function 'GivenClp_destroy' could not destroy a structure\n" );
		fprintf( stderr, "'GivenCommandLineParameters' that has not been created before!\n" );
		AbortProgram;
	}
}

/** Displays a @ref GivenCommandLineParameters structure.
*
* The structure @ref GivenCommandLineParameters the pointer @c givenClpPtr refers to is displayed
* on the FILE* @c outputStream.
*
* @param givenClpPtr pointer on the structure @ref GivenCommandLineParameters that should be displayed
* @param outputStream FILE* on the stream the output should be sent to
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
* @version 24/11/2003 - Uli Fechner - adjusted according to the needs of retroflux
* @version 29/01/2004 - Tina Grabowski - added support for the member @c sOptionGiven
*/
void GivenClp_display( const Given_CLP_Ptr givenClpPtr, FILE* outputStream )
{
	fprintf( outputStream, "oOptionGiven = %d\n", givenClpPtr->oOptionGiven );
	fprintf( outputStream, "rOptionGiven = %d\n", givenClpPtr->rOptionGiven );
	fprintf( outputStream, "fOptionGiven = %d\n", givenClpPtr->fOptionGiven );
	fprintf( outputStream, "sOptionGiven = %d\n", givenClpPtr->sOptionGiven );
}

/** The option @c option of a @ref GivenCommandLineParameters structure is set to @c boolean_value.
*
* The parameter @c boolean_value could either have the value @ref BOOLEAN_TRUE or @ref BOOLEAN_FALSE.
*
* @param givenClpPtr pointer on the structure @ref GivenCommandLineParameters
* @param option character of the option to be set (o, r)
* @param boolean_value one of the two defines @ref BOOLEAN_TRUE or @ref BOOLEAN_FALSE
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
* @version 24/11/2003 - Uli Fechner - adjusted according to the needs of retroflux
* @version 29/01/2004 - Tina Grabowski - added support for the member @c sOptionGiven
*/
void GivenClp_setOption( const Given_CLP_Ptr givenClpPtr, const int option, const int boolean_value )
{
	switch( option )
	{
	case 'o':
		givenClpPtr->oOptionGiven = boolean_value;
		break;
	case 'r':
		givenClpPtr->rOptionGiven = boolean_value;
		break;
	case 'f':
		givenClpPtr->fOptionGiven = boolean_value;
		break;
	case 's':
		givenClpPtr->sOptionGiven = boolean_value;
		break;
	default:
		fprintf(stderr, "\n\nERROR: Function 'GivenClp_setOption' recieved wrong argument 'option'.\n" );
		fprintf(stderr, "'%c' is not a valid argument for this function!\n", option );
		AbortProgram;
		break;
	}
}

/** The current value of option @c option of a @ref GivenCommandLineParameters structure is returned.
*
* The return value @c boolean_value could either have the value @ref BOOLEAN_TRUE or @ref BOOLEAN_FALSE.
*
* @param givenClpPtr pointer on the structure @ref GivenCommandLineParameters
* @param option character of the option to be set (o, r)
* @retval int one of the two defines @ref BOOLEAN_TRUE or @ref BOOLEAN_FALSE
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
* @version 24/11/2003 - Uli Fechner - adjusted according to the needs of retroflux
* @version 29/01/2004 - Tina Grabowski - added support for the member @c sOptionGiven
*/
int GivenClp_getOption( const Given_CLP_Ptr givenClpPtr, const int option )
{
	switch( option )
	{
	case 'o':
		return givenClpPtr->oOptionGiven;
		break;
	case 'r':
		return givenClpPtr->rOptionGiven;
		break;
	case 'f':
		return givenClpPtr->fOptionGiven;
		break;
	case 's':
		return givenClpPtr->sOptionGiven;
		break;
	default:
		fprintf(stderr, "\n\nERROR: Function 'GivenClp_getOption' recieved wrong argument 'option'.\n" );
		fprintf(stderr, "'%c' is not a valid argument for this function.\n", option );
		AbortProgram;
		break;
	}
}
